; Author:      Roy Klein Gebbinck (www.b-k-g.nl)
; Version:     20180418: First version.
; Created for: https://forum.bricsys.com/discussion/32554/xnotifytime-less-than-one-minute

; 20180418
; Return value: List of loaded xref definition objects.
(defun BKG_ReloadXrefs_XrefList ( / ret)
  (vlax-for def (vla-get-blocks (vla-get-activedocument (vlax-get-acad-object)))
    (if
      (and
        (= -1 (vlax-get def 'isxref))
        (=  0 (vlax-get def 'isunloaded)) ; Note: vla-get-isunloaded not available before V16.
      )
      (setq ret (cons def ret))
    )
  )
  (reverse ret)
)

; 20180418
; Return value: List. Format: ((<xref def object> <systime list>) (<xref def object> <systime list>) ...)
(defun BKG_ReloadXrefs_DataList (objLst)
  (vl-remove
    nil
    (mapcar
      '(lambda (obj / time)
        (if (setq time (vl-file-systime (vla-get-path obj))) ; Nil if file is not found.
          (list obj time)
        )
      )
      objLst
    )
  )
)

; 20180418
; Side effect: The global variable *BKG_ReloadXrefs_dataLst* is updated.
; Return value: List of xref def objects to be reloaded.
(defun BKG_ReloadXrefs_Compare ( / curDataLst fnd objLst ret)
  (setq objLst (BKG_ReloadXrefs_XrefList))
  (cond
    ((not objLst)
      (setq *BKG_ReloadXrefs_dataLst* nil)
      nil
    )
    ((not *BKG_ReloadXrefs_dataLst*)
      (setq *BKG_ReloadXrefs_dataLst* (BKG_ReloadXrefs_DataList objLst))
      nil
    )
    (T
      (setq curDataLst (BKG_ReloadXrefs_DataList objLst))
      (foreach sub curDataLst
        (if
          (and
            (setq fnd (assoc (car sub) *BKG_ReloadXrefs_dataLst*))
            (not (equal sub fnd))
          )
          (setq ret (cons (car sub) ret))
        )
      )
      (setq *BKG_ReloadXrefs_dataLst* curDataLst)
      (reverse ret)
    )
  )
)

; 20180418
; Purpose: Reactor callback function.
; Side effect 1: The global variable *BKG_ReloadXrefs_reloadLst* is updated.
; Side effect 2: The BKG_ReloadXrefs_Reload command is sent.
; Return value: Suppressed.
(defun BKG_ReloadXrefs_CallBack (rea lst / objLst)
  (if
    (and
      (not (vl-position (car lst) '("SAVE" "SAVEAS" "SAVEASR12" "QSAVE")))
      (setq objLst (BKG_ReloadXrefs_Compare))
    )
    (progn
      (setq *BKG_ReloadXrefs_reloadLst* (append *BKG_ReloadXrefs_reloadLst* objLst))
      (vla-sendcommand (vla-get-activedocument (vlax-get-acad-object)) "BKG_ReloadXrefs_Reload ")
    )
  )
)

; 20180418
; Purpose: Command function to reload the xrefs in *BKG_ReloadXrefs_reloadLst*.
; Side effect: The global variable *BKG_ReloadXrefs_reloadLst* is updated.
; Return value: None.
(defun c:BKG_ReloadXrefs_Reload ( / doneLst)
  (if *BKG_ReloadXrefs_reloadLst*
    (progn
      (foreach obj *BKG_ReloadXrefs_reloadLst*
        (if (not (vl-position obj doneLst))
          (progn
            (princ (strcat "\nBKG_ReloadXrefs: reloading \"" (vla-get-path obj) "\"... "))
            (vla-reload obj)
            (setq doneLst (cons obj doneLst))
          )
        )
      )
      (setq *BKG_ReloadXrefs_reloadLst* nil)
    )
  )
  (princ)
)

; 20180418
; Purpose: Set up the reactors.
; Return value: none.
(defun BKG_ReloadXrefs_OnLoad ()
  ;; Remove existing reactors:
  (mapcar
    '(lambda (rea)
      (if (= "BKG_ReloadXrefs" (vlr-data rea))
        (vlr-remove rea)
      )
    )
    (apply 'append (mapcar 'cdr (vlr-reactors)))
  )
  ;; Create new reactors:
  (vlr-docmanager-reactor
    "BKG_ReloadXrefs"
    '(
      (:vlr-documentbecamecurrent . BKG_ReloadXrefs_CallBack)
    )
  )
  (vlr-command-reactor
    "BKG_ReloadXrefs"
    '(
      (:vlr-commandcancelled . BKG_ReloadXrefs_CallBack)
      (:vlr-commandended     . BKG_ReloadXrefs_CallBack)
      (:vlr-commandfailed    . BKG_ReloadXrefs_CallBack)
    )
  )
  (princ)
)

(BKG_ReloadXrefs_OnLoad)
(princ)